package de.pincservices.pdf;


import com.itextpdf.text.Document
import com.itextpdf.text.pdf.PdfWriter
import de.pincservices.pdf.renderer.AbstractRenderer
import de.pincservices.pdf.renderer.FlyingSaucerPdfRenderer
import org.apache.commons.lang.StringUtils
import org.slf4j.Logger
import org.slf4j.LoggerFactory

/**
 * Main class to create a pdf from different sources.
 */
public class PdfRenderer extends AbstractRenderer {

    private static final Logger LOGGER = LoggerFactory.getLogger(PdfRenderer.class);
    private static final String BASE_PDF_KEY = "basePdf";
    private static final String BACKGROUND_REPEATING_KEY = "backgroundRepeating";

    private List<RenderingPreProcessor> _preProcessors = new ArrayList<RenderingPreProcessor>();
    private Renderer _renderer = new FlyingSaucerPdfRenderer();

    private Configuration _configuration;

    public PdfRenderer(String basePdfFilename, boolean repeating) {
        _configuration = new Configuration();
        _configuration.setProperty(BASE_PDF_KEY, basePdfFilename);
        _configuration.setProperty(BACKGROUND_REPEATING_KEY, repeating);
    }

    public void createFromFile(File file, OutputStream outputStream) throws IOException {
        def baseUrl = file?.parentFile?.absoluteFile?.toURI()?.toString()
        def content = file?.getText()
        createFromContent([content], outputStream, baseUrl)
    }

    @Override
    public void createFromContent(List<String> pages, OutputStream outputStream, String externalResourcesBaseUrl) {
        if (_renderer == null) {
            throw new IllegalStateException("Error: No renderer for PdfRenderer defined.");
        }

        final List<String> contentPages = getPreprocessedPages(pages);

        if (hasBackgroundPdf()) {
            renderDocumentWithBackground(contentPages, externalResourcesBaseUrl, outputStream)
        } else {
            renderDocumentWithoutBackground(contentPages, externalResourcesBaseUrl, outputStream);
        }
    }

    public void appendPreProcessor(RenderingPreProcessor preProcessor) {
        if (preProcessor == null) {
            throw new IllegalArgumentException("preprocessor must not be null");
        }
        _preProcessors.add(preProcessor);
    }

    public void removePreProcessor(RenderingPreProcessor preProcessor) {
        if (preProcessor != null) {
            _preProcessors.remove(preProcessor);
        }
    }

    public List<RenderingPreProcessor> getPreProcessors() {
        return Collections.unmodifiableList(_preProcessors);
    }

    public Renderer getRenderer() {
        return _renderer;
    }

    public void setRenderer(Renderer renderer) {
        if (renderer == null) {
            throw new IllegalArgumentException("renderer must not be null");
        }
        _renderer = renderer;
    }

    private def renderDocumentWithBackground(ArrayList<String> contentPages, String externalResourcesBaseUrl, OutputStream outputStream) {
        // Piped streams do not work here, unfortunately (results in idle waiting).
        final ByteArrayOutputStream temporaryOutputStream = new ByteArrayOutputStream();
        _renderer.createFromContent(contentPages, temporaryOutputStream, externalResourcesBaseUrl);

        // start merging the PDFs ...
        final ByteArrayInputStream foregroundInputStream = new ByteArrayInputStream(temporaryOutputStream.toByteArray());

        try {
            def repeatBackground = _configuration.getProperty(BACKGROUND_REPEATING_KEY).is(Boolean.TRUE);
            def backgroundLayer = new DocumentLayer(_configuration.getProperty(BASE_PDF_KEY).asString(), repeatBackground);
            def foregroundLayer = new DocumentLayer(foregroundInputStream, false);
            def documentLayers = [backgroundLayer, foregroundLayer];

            merge(documentLayers, outputStream);
        } catch (Exception e) {
            throw new RuntimeException("Error occured while merging documents.", e);
        }
    }

    private def renderDocumentWithoutBackground(ArrayList<String> contentPages, String externalResourcesBaseUrl, OutputStream outputStream) {
        _renderer.createFromContent(contentPages, outputStream, externalResourcesBaseUrl)
    }

    private boolean hasBackgroundPdf() {
        return StringUtils.isNotEmpty(_configuration.getProperty(BASE_PDF_KEY).asString())
    }

    private List<String> getPreprocessedPages(List<String> pages) {
        List<String> contentPages = new ArrayList<String>(pages);
        _preProcessors?.each { processor ->
            contentPages = processor.processContent(contentPages)
        }
        return contentPages;
    }

    /**
     * Merge all given documentLayers into the outputFile. The documentLayers are layered, not concatenated.
     */
    private void merge(Collection<DocumentLayer> documentLayers, OutputStream outputStream) {
        // Get maximum page count.
        def pageCount = documentLayers?.collect { it.getNumberOfPages() }?.max()
        pageCount = pageCount != null ? pageCount : 0

        def document = new Document();
        // output stream could also be a ByteArrayOutputStream for later processing:
        def writer = PdfWriter.getInstance(document, outputStream);
        document.open();

        (1..pageCount).each {
            document.newPage();
            documentLayers?.each { layer -> layer.writeCurrentPage(writer) }
        }

        document.close();
    }
}
