package de.pincservices.pdf;

import com.itextpdf.text.pdf.PdfReader;
import de.pincservices.pdf.preprocessor.VelocityEvaluator;
import de.pincservices.template.CustomMacroDirective;
import de.pincservices.template.TemplateDirective;
import de.pincservices.template.velocity.AbstractVelocityTemplateDirective;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang.time.DateUtils;
import org.apache.velocity.context.InternalContextAdapter;
import org.apache.velocity.exception.MethodInvocationException;
import org.apache.velocity.exception.ParseErrorException;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.apache.velocity.runtime.parser.node.Node;
import org.apache.velocity.tools.generic.DateTool;
import org.apache.velocity.tools.generic.NumberTool;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.Writer;
import java.text.SimpleDateFormat;
import java.util.*;

import static org.hamcrest.CoreMatchers.is;
import static org.junit.Assert.assertThat;

public class PdfRendererTest {

    @Before
    public void removeCreatedFile() {
        File file = new File("test.pdf");
        if (file.exists()) {
            assertThat(FileUtils.deleteQuietly(file), is(true));
        }
    }

    @Test
    public void testCreateFromContentWithoutBackground() throws Exception {
        final PdfRenderer renderer = new PdfRenderer(null, false);

        final StringBuilder builder = new StringBuilder();
        builder.append("<html><body>Tabletest2<br /><table border=\"1\">");
        for (int i = 0; i < 20; i++) {
            builder.append("<tr><td>line ");
            builder.append(i);
            builder.append("</td></tr>");
        }
        builder.append("</table></body></html>");

        String[] pages = new String[2];
        pages[0] = builder.toString();
        pages[1] = "<html><body>Folgeseite</body></html>";
        renderer.createFromContent(Arrays.asList(pages), "test.pdf", null);

        File createdFile = new File("test.pdf");
        assertThat(createdFile.exists(), is(true));
        File comparisonFile = new File("src/test/resources/pdf/flyingsaucer_compare.pdf");
        assertThat(createdFile.length(), is(comparisonFile.length()));
        PdfReader reader1 = new PdfReader(createdFile.getAbsolutePath());
        PdfReader reader2 = new PdfReader(comparisonFile.getAbsolutePath());
        assertThat(reader1.getNumberOfPages(), is(reader2.getNumberOfPages()));
        for (int i = 0; i < reader1.getNumberOfPages(); i++) {
            final byte[] content1 = reader1.getPageContent(i);
            final byte[] content2 = reader2.getPageContent(i);
            assertThat(Arrays.equals(content1, content2), is(true));
        }
    }

    @Test
    public void testLayeringWithoutRepeating() throws Exception {
        final PdfRenderer renderer = new PdfRenderer("src/test/resources/pdf/background.pdf", false);

        String[] pages = new String[2];
        pages[0] = "<html><body>Seite 1</body></html>";
        pages[1] = "<html><body>Folgeseite</body></html>";
        renderer.createFromContent(Arrays.asList(pages), "test.pdf", null);

        File createdFile = new File("test.pdf");
        assertThat(createdFile.exists(), is(true));
        File comparisonFile = new File("src/test/resources/pdf/layer_nonrepeating_compare.pdf");
        assertThat(createdFile.length(), is(comparisonFile.length()));
        PdfReader reader1 = new PdfReader(createdFile.getAbsolutePath());
        PdfReader reader2 = new PdfReader(comparisonFile.getAbsolutePath());
        assertThat(reader1.getNumberOfPages(), is(reader2.getNumberOfPages()));
        for (int i = 0; i < reader1.getNumberOfPages(); i++) {
            final byte[] content1 = reader1.getPageContent(i);
            final byte[] content2 = reader2.getPageContent(i);
            assertThat(Arrays.equals(content1, content2), is(true));
        }
    }

    @Test
    public void testLayeringWithRepeating() throws Exception {
        final PdfRenderer renderer = new PdfRenderer("src/test/resources/pdf/background.pdf", true);
        String[] pages = new String[2];
        pages[0] = "<html><body>Seite 1</body></html>";
        pages[1] = "<html><body>Folgeseite</body></html>";
        renderer.createFromContent(Arrays.asList(pages), "test.pdf", null);

        File createdFile = new File("test.pdf");
        assertThat(createdFile.exists(), is(true));
        File comparisonFile = new File("src/test/resources/pdf/layer_repeating_compare.pdf");
        assertThat(createdFile.length(), is(comparisonFile.length()));
        PdfReader reader1 = new PdfReader(createdFile.getAbsolutePath());
        PdfReader reader2 = new PdfReader(comparisonFile.getAbsolutePath());
        assertThat(reader1.getNumberOfPages(), is(reader2.getNumberOfPages()));
        for (int i = 0; i < reader1.getNumberOfPages(); i++) {
            final byte[] content1 = reader1.getPageContent(i);
            final byte[] content2 = reader2.getPageContent(i);
            assertThat(Arrays.equals(content1, content2), is(true));
        }
    }

    @Test
    public void testRenderingWithEvaluator() throws Exception {
        final PdfRenderer renderer = new PdfRenderer("src/test/resources/pdf/background.pdf", true);

        final Map<String, Object> parameters = new HashMap<String, Object>();
        parameters.put("testExpression", "kleiner Velocity-Ersetzung");
        parameters.put("user", new User());
        VelocityEvaluator evaluator = new VelocityEvaluator(Configuration.forMap(parameters));
        renderer.appendPreProcessor(evaluator);

        String[] pages = new String[3];
        pages[0] = "<html><body>Seite 1</body></html>";
        pages[1] = "<html><body>Folgeseite mit ${testExpression} (testExpression).</body></html>";
        pages[2] = "<html><body>Folgeseite für ${user.name} (user.name).</body></html>";
        renderer.createFromContent(Arrays.asList(pages), "test.pdf", null);

        File createdFile = new File("test.pdf");
        assertThat(createdFile.exists(), is(true));
        File comparisonFile = new File("src/test/resources/pdf/layer_repeating_with_expression_compare.pdf");
        assertThat(createdFile.length(), is(comparisonFile.length()));
        PdfReader reader1 = new PdfReader(createdFile.getAbsolutePath());
        PdfReader reader2 = new PdfReader(comparisonFile.getAbsolutePath());
        assertThat(reader1.getNumberOfPages(), is(reader2.getNumberOfPages()));
        for (int i = 0; i < reader1.getNumberOfPages(); i++) {
            final byte[] content1 = reader1.getPageContent(i);
            final byte[] content2 = reader2.getPageContent(i);
            assertThat(Arrays.equals(content1, content2), is(true));
        }
    }

    @Test
    public void testRenderingFromFileWithEvaluator() throws Exception {
        final PdfRenderer renderer = new PdfRenderer(null, false);

        final Map<String, Object> parameters = new HashMap<String, Object>();
        parameters.put("taxInterest", 0.19);
        parameters.put("numberTool", new NumberTool());
        parameters.put("dateTool", new DateTool());
        parameters.put("customMacro", new CustomMacroDirective());
        parameters.put("user", new User());
        VelocityEvaluator evaluator = new VelocityEvaluator(Configuration.forMap(parameters));
        renderer.appendPreProcessor(evaluator);

        final FileOutputStream outputStream = new FileOutputStream("test.pdf");
        renderer.createFromFile(new File("src/test/resources/templates/template.xml"), outputStream);
        //outputStream.flush();
        outputStream.close();
        File createdFile = new File("test.pdf");
        assertThat(createdFile.exists(), is(true));
        File comparisonFile = new File("src/test/resources/pdf/flyingsaucer_from_file_compare.pdf");
        Assert.assertEquals(comparisonFile.length(), createdFile.length());
        PdfReader reader1 = new PdfReader(createdFile.getAbsolutePath());
        PdfReader reader2 = new PdfReader(comparisonFile.getAbsolutePath());
        assertThat(reader1.getNumberOfPages(), is(reader2.getNumberOfPages()));
        for (int i = 0; i < reader1.getNumberOfPages(); i++) {
            final byte[] content1 = reader1.getPageContent(i);
            final byte[] content2 = reader2.getPageContent(i);
            assertThat(Arrays.equals(content1, content2), is(true));
        }
    }

    public class User {

        private Address _address = new Address();
        private String _name = "Testuser";
        private Date _birthDate;

        public User() {
            final Calendar calendar = Calendar.getInstance();
            calendar.set(2011,0, 2);
            _birthDate = calendar.getTime();
        }
        public Address getAddress() {
            return _address;
        }

        public String getName() {
            return _name;
        }

        public Date getBirthDate() {
            return _birthDate;
        }
    }

    public class Address {

        private String _street = "FooStreet";
        private String _streetNumber = "123";
        private String _city = "BarTown";
        private String _zipCode = "12345";

        public String getStreet() {
            return _street;
        }

        public String getStreetNumber() {
            return _streetNumber;
        }

        public String getCity() {
            return _city;
        }

        public String getZipCode() {
            return _zipCode;
        }
    }
}
